#************************* NSIS Headers ****************************************#
;----- Include Modern UI ---------------------#
!include "MUI2.nsh"

;----- Include Uninstaller --------------------#
; - This is from http://nsis.sourceforge.net/Uninstall_from_NSIS_log
!include "uninstallfromlog.nsh" 
;----- Helper headers --------------------#
!include WordFunc.nsh
!include LogicLib.nsh


#****************************General definitions *******************************#
!define ORG_NAME "Battlezone II"
!define APP_NAME "MyMod's App_Name"
!define SUITE_NAME "MyModSuiteName"
!define APP_SHORTNAME "MyMod"
!define APP_SDN "Battlezone II - MyMod"

BrandingText "ModMaker's BrandingText"

ShowInstDetails show
ShowUninstDetails show

;Vista redirects $SMPROGRAMS to all users without this
RequestExecutionLevel admin

!define EXE_NAME bzone.exe
!define EXE_NAME2 bz2edit.exe

!define MUI_ABORTWARNING

!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP "header.bmp"
!define MUI_HEADERIMAGE_BITMAP_NOSTRETCH
!define MUI_HEADER_TRANSPARENT_TEXT

!define MUI_HEADERIMAGE_UNBITMAP "un_header.bmp"
!define MUI_HEADERIMAGE_UNBITMAP_NOSTRETCH

!define MUI_WELCOMEFINISHPAGE_BITMAP "wizard.bmp"
!define MUI_UNWELCOMEFINISHPAGE_BITMAP "un_wizard.bmp"

; Set a custom icon for the install exe
!define MUI_ICON "install.ico"
!define MUI_UNICON "un_install.ico"


#********************** Modern UI Pages ****************************************#

!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_DIRECTORY

!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_WELCOME
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

#----- Modern UI Languages -------------------#
!insertmacro MUI_LANGUAGE "English"

#----- Installer options ---------------------#
Name "${APP_NAME}"

!ifndef VERSION_TAG
	!define VERSION_TAG "1.0.0.0"
!endif

!define OUTFILE_NAME ${APP_SHORTNAME}-installer_${VERSION_TAG}.exe

!ifdef OUTFILE_PATH
	!define OUTFILE_FULLPATH "${OUTFILE_PATH}\${OUTFILE_NAME}"
!else
	!define OUTFILE_FULLPATH ".\${OUTFILE_NAME}"
!endif
OutFile ${OUTFILE_FULLPATH}

#----- Target Location -----------------------#
InstallDir "$PROGRAMFILES"


#----- Windows Uninstall ---------------------#
; Pathname for the uninstall executable
!define UNINST_EXE "$INSTDIR\Uninstall ${SUITE_NAME}${APP_SHORTNAME}.exe"

;The following two definitions add uninstall information to Add/Remove Programs.
;   Uninstall logging will use these definitions.
;   You may use these definitions also, when you want to:
;   Set up the InstallDirRegKey, Store the language selection, Store Start Menu folder etc.
!define INSTDIR_REG_ROOT "HKLM"
!define INSTDIR_REG_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${SUITE_NAME}${APP_SHORTNAME}"

InstallDirRegKey ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "InstallDir"

#***************************** Installer Sections ******************************#

# ----- Base installation Section -------------#
Section "!MyMod Files" main
	LogSet On
		SetOutPath $INSTDIR
		SetOverwrite try
; !!! ModAuthors: BEGIN fill in this section with files you want to install.
		File mymod.cfg
; This pulls in MyModAddonDir and all subdirs, except any .svn folders
		File /r /x *.svn MyModAddonDir
; Uncomment this when you make mymod.pak
;		File mymod.pak

; !!! ModAuthors: END fill in this section with files you want to install.
		goto end_main
	access_error:
		!define ACCESS_ERROR "You do not have the appropriate user rights to install this BZ2 mod.  Please contact your system administrator for assistance."
		MessageBox MB_OK "${ACCESS_ERROR}"
		Abort "${ACCESS_ERROR}"
	end_main:
	LogSet Off
SectionEnd


# ----- Start Menu Shortcuts ------------------#
Section "Start Menu Shortcuts" shortcuts
; Main Menu
	LogSet On
	SetShellVarContext all
	CreateDirectory '$SMPROGRAMS\Battlezone II\${SUITE_NAME}'
	CreateShortCut "$SMPROGRAMS\Battlezone II\${SUITE_NAME}\BZ2 ${APP_SHORTNAME}.lnk" "$INSTDIR\${EXE_NAME}" "-config mymod.cfg"
	CreateShortCut "$SMPROGRAMS\Battlezone II\${SUITE_NAME}\BZ2Edit ${APP_SHORTNAME}.lnk" "$INSTDIR\${EXE_NAME2}" "-config mymod.cfg"
	CreateShortCut "$SMPROGRAMS\Battlezone II\${SUITE_NAME}\Uninstall ${APP_SHORTNAME}.lnk" '${UNINST_EXE}'
	
	SetShellVarContext current
	LogSet Off
SectionEnd

;----- Base installation -----------------------#
; This ensures that some kind of uninstall.exe gets written.
; It handles the case where the user chooses to install nothing.
Section "-Base"
	LogSet On
		SetOutPath $INSTDIR
;		File ${REDIST_DIR}\version.txt

; Write the uninstall keys for Windows
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "InstallDir" "$INSTDIR"
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "DisplayIcon" "${UNINST_EXE}"
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "DisplayName" "${SUITE_NAME}: ${APP_SDN}"
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "DisplayVersion" "${VERSION_TAG}"
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "NoModify" "1"
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "NoRepair" "1"

; Same as create shortcut, need to use ${UNINST_EXE} instead of anything else.
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "UninstallString" "${UNINST_EXE}"
		WriteRegStr ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "QuietUninstallString" "${UNINST_EXE} /S"
		WriteRegDWORD ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "NoModify" 1
		WriteRegDWORD ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "NoRepair" 1

		WriteUninstaller "${UNINST_EXE}"
	LogSet Off
SectionEnd

#----- Uninstaller -----------------------------#
Section Uninstall
	ReadRegStr $INSTDIR ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "InstallDir"

	Call un.CreateLogFromFile
	Call un.RemoveDirectoriesFromLog

	SetShellVarContext all
	Delete "$DESKTOP\${SUITE_NAME} ${APP_SHORTNAME}.lnk"
	Delete "$SMPROGRAMS\${ORG_NAME}\${SUITE_NAME}\${APP_SDN}\${SUITE_NAME} ${APP_SHORTNAME}.lnk"
;	Delete "$SMPROGRAMS\${ORG_NAME}\${SUITE_NAME}\${APP_SDN}\Release Notes.lnk"
	Delete "$SMPROGRAMS\${ORG_NAME}\${SUITE_NAME}\${APP_SDN}\Uninstall ${APP_SHORTNAME}.lnk"

; Delete folders. This shouldn't be done as a mod.
;	RmDir "$SMPROGRAMS\${ORG_NAME}\${SUITE_NAME}\${APP_SDN}"
;	RmDir "$SMPROGRAMS\${ORG_NAME}\${SUITE_NAME}"
;	RmDir "$SMPROGRAMS\${ORG_NAME}"
	SetShellVarContext current

	DeleteRegKey /ifempty ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}"

	Delete "${UNINST_EXE}"
	Delete "$INSTDIR\install.log"
SectionEnd

#************************* Text Descriptions ***************************************#
;----- English ----------------------------------#
LangString DESC_main ${LANG_ENGLISH} "MyMod's DESC_main configurations"
LangString DESC_shortcuts ${LANG_ENGLISH} "Windows Start Menu Shortcuts"

;----- Modern UI Definitions --------------------#
!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
!insertmacro MUI_DESCRIPTION_TEXT ${main} $(DESC_main)
!insertmacro MUI_DESCRIPTION_TEXT ${shortcuts} $(DESC_shortcuts)
!insertmacro MUI_FUNCTION_DESCRIPTION_END

;----- Windows File Properties Version Tab ------#
VIProductVersion "${VERSION_TAG}"
VIAddVersionKey /LANG=${LANG_ENGLISH} "ProductName" "MyMod's ProductName"
VIAddVersionKey /LANG=${LANG_ENGLISH} "CompanyName" "MyCompanyName"
;VIAddVersionKey /LANG=${LANG_ENGLISH} "LegalTrademarks" "Battlezone II is a trademark of Activision, Atari, PandemicStudios"
VIAddVersionKey /LANG=${LANG_ENGLISH} "LegalCopyright" " 2011 MyMod Author"
VIAddVersionKey /LANG=${LANG_ENGLISH} "FileDescription" "MyMod FileDescription"
VIAddVersionKey /LANG=${LANG_ENGLISH} "FileVersion" "${VERSION_TAG}"

#***************************** Functions ******************************#
Function .onInit
; Read where 1.3 was last installed, use that as default install dir
	ReadRegStr $R0 HKLM "SOFTWARE\Activision\Battlezone II" "Install121Dir"
	${If} $R0 != ""
		StrCpy $INSTDIR $R0
	${EndIf}
; Messagebox MB_OK|MB_ICONINFORMATION "Got $INSTDIR"
	Call CheckInstallStatus
FunctionEnd

Function CheckInstallStatus
;===Registers===
;$R0 uninstall exe
;$R1 existing installed version
;$R2 version comparison result
;$R3 uninstall exe-path
	Push $R0
	Push $R1
	Push $R2
	Push $R3

	checkGCE:
	ReadRegStr $R0 ${INSTDIR_REG_ROOT} "${OLD_INSTDIR_REG_KEY}" "UninstallString"
	IfErrors checkMod 
	ReadRegStr $R1 ${INSTDIR_REG_ROOT} "${OLD_INSTDIR_REG_KEY}" "DisplayVersion"
	goto compare

	checkMod:
	ReadRegStr $R0 ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "UninstallString"
	IfErrors end
	ReadRegStr $R1 ${INSTDIR_REG_ROOT} "${INSTDIR_REG_KEY}" "DisplayVersion"

	compare:
	${VersionCompare} ${VERSION_TAG} $R1 $R2
	IntCmp $R2 1 upgrade equal downgrade

	upgrade:
	IfSilent uninst
	MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "An older version of ${APP_NAME} is already \
		installed ($R1).$\n$\nClick 'OK' to upgrade to ${VERSION_TAG} or 'Cancel' to abort the \
		installation." IDOK uninst
	Abort
	equal:
	IfSilent +2
	MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "${APP_NAME} $R1 is already installed.$\n$\nClick \
		'OK' to reinstall or 'Cancel' to abort the installation." IDOK uninst
	Abort
	downgrade:
	IfSilent +2
	MessageBox MB_OK|MB_ICONEXCLAMATION "A newer version of ${APP_NAME} is already installed \
		($R1).$\n$\nThis installation will now terminate.$\nIf you wish to downgrade, you may \
			uninstall the current version and rerun this install wizard."
	Abort

	;Run the uninstaller
	uninst:
	${GetParent} $R0 $R3
	HideWindow
	ClearErrors
	IfSilent 0 +3
	ExecWait '"$R0 /S" _?=$R3'
	goto end
	ExecWait '"$R0" _?=$R3'
	IfErrors 0 end
	Abort "An error was encountered during the uninstall process. Press 'Cancel' to leave the installer."

	end:
	Pop $R3
	Pop $R2
	Pop $R1
	Pop $R0
FunctionEnd
